import numpy as np
import matplotlib.pyplot as plt
from math import floor
from scipy.optimize import curve_fit

def _expfit(t, *args):
    c = args
    return np.exp(-t / c)

# fit the autocorrelation function to an exponential (to compute correlation time)
def fit_autocorr_fcn(t, a_auto):
    fitted_params, _ = curve_fit(_expfit, t, a_auto, p0=[1])
    t_to_plot = np.linspace(np.min(t), np.max(t), 1000)
    plt.plot(t_to_plot, _expfit(t_to_plot, *fitted_params), '--', label='Fit')
    return fitted_params

# compute the autocorrelation function of some data series a
# to some maximum time t_max
def compute_autocorr_fcn(a, t_max=-1):
    n_steps = len(a)
    if t_max == -1:
        t_max = floor(n_steps / 2.)
    a_auto = np.zeros(t_max)
    for i_step in range(t_max):
        if i_step == 0:
            a_auto[i_step] = np.mean(a * a)
        else:
            a_auto[i_step] = np.mean(a[:-i_step] * a[i_step:])
    a_mean = np.mean(a)
    a_auto = (a_auto - a_mean * a_mean) / (np.mean(a * a) - a_mean * a_mean)
    return a_auto

if __name__ == '__main__':
    
    # compute autocorrelation function for end-to-end distance trajectory
    fname = 'ree_seq11111n1n1n1n1n111111_T1.0.txt'
    ree = np.loadtxt(fname)
    ree_auto = compute_autocorr_fcn(ree, t_max=100)
    t = np.arange(len(ree_auto)) * 10
    plt.plot(t, ree_auto, label='Simulated')
    print(fit_autocorr_fcn(t, ree_auto))
    plt.legend(edgecolor='None')
    plt.xlim([0, 1000])
    plt.xlabel(r'Number of steps, $n$')
    plt.ylabel(r'$C_{Ree}(n)$')
    plt.savefig('ree_autocorr.png', dpi=200, bbox_inches='tight')

